/*
  ContentArea.js

  This governs the actual content displayed by the App. I pushed this down just to further illustrate the use of a container component.

  We have now given this component responsibility for managing the data and coordianting the views.
*/

import React, { Component } from 'react';

import Article from './Article.js';
import IndexBar from './IndexBar.js';
import Editor from './Editor.js';
import StyledButton from './StyledButton.js';
import styled from 'styled-components';


//const SERVER = 'http://basin.cs.middlebury.edu:3042';
const SERVER = 'http://localhost:3001';


const ButtonBar = styled.div`
  margin:40px;
`;


class ContentArea extends Component{
  constructor(props){
    super(props);
    this.state={
      mode: 'load',
      currentArticle: null
    };

    fetch(`${SERVER}/articles`)
    .then((response)=>{
      if (response.ok){
        return response.json();
      }
    })
    .then((data)=>{
      this.setState({collection: data, mode:'view'});
    });



  }

  /*
  This function handles the result of the editor resigning control.
  */
  handleEditorReturn(newArticle){
    if (newArticle){ // not a cancel
      let request;
      if (this.state.currentArticle){
        // this is a replacement, remove the old article

        // merge in the other properties of the old version of this article
        newArticle = Object.assign({}, this.state.currentArticle, newArticle);

        request = new Request(SERVER+'/articles/'+newArticle._id, {
    	    method: 'PUT',
  	      body:JSON.stringify(newArticle),
          headers: new Headers({
            'Accept': 'application/json',
            'Content-Type': 'application/json'
          })
        });
      } else{
        // this is a new article
        request = new Request(SERVER+'/articles/', {
    	    method: 'POST',
  	      body:JSON.stringify(newArticle),
          headers: new Headers({
            'Accept': 'application/json',
            'Content-Type': 'application/json'
          })
        });
      }

      fetch(request)
      .then((response)=>{
        if (response.ok){
          return response.json();
        }
        else{
          throw new Error('Rejected by server');
        }
      })
      .then((newArticle)=>{
        // we are trying to merge the new article into the collection
        // this will not work if this is a new document, but we still want the copy of
        // the array
        let replaced = false;
        const modifiedCollection = this.state.collection.map((article) =>{
          if (article._id === newArticle._id){
            replaced = true;
            return newArticle;
          }else{
            return article;
          }
        });

        if (! replaced){
          // this was a new article, just add it on the end of the new list
          modifiedCollection.push(newArticle);
        }

        this.setState({collection:modifiedCollection, currentArticle: newArticle, mode: 'view'});
      })
      .catch((err)=>{
        // something went wrong, bail out
        alert('Unable to save article: ' + err.message);
        this.setState({mode:'view'});
      });
    }else{
      // cancel, just switch to the browsing view
      this.setState({mode:'view'});
    }
  }

  removeArticle(oldArticle){
    const request = new Request(SERVER+'/articles/'+oldArticle._id, {
      method: 'DELETE',
      headers: new Headers({
        'Accept': 'application/json',
        'Content-Type': 'application/json'
      })
    });

    fetch(request)
    .then((response)=>{
      if (response.ok){
        const modifiedCollection = this.state.collection.filter((article)=>{
          return article._id !== oldArticle._id;
        });
        this.setState({collection:modifiedCollection, currentArticle: null});
      }
    })
  }

  render(){
    if (this.state.mode === 'load'){
      return (<h2>Loading...</h2>);
    }else if (this.state.mode === 'view'){
      // create the IndexBar
      let indexBar = (<IndexBar collection={this.state.collection} currentArticle={this.state.currentArticle} select={(article)=>this.setState({currentArticle:article})}/>);

      // create our buttons, we will pick which ones to show based on the// state of curentArticle
      const newButton = (<StyledButton onClick={()=>this.setState({mode:'edit', currentArticle:null})}>New Article</StyledButton>);
      const editButton = (<StyledButton onClick={()=>this.setState({mode:'edit'})}>Edit article</StyledButton>);
      const deleteButton = (<StyledButton onClick={()=>this.removeArticle(this.state.currentArticle)}>Delete article</StyledButton>);


      // create the article if we have a current article
      let contents ;
      if (this.state.currentArticle){
        contents = (<div>
          <Article article={this.state.currentArticle}/>
          <ButtonBar>
            {newButton}
            {editButton}
            {deleteButton}
          </ButtonBar>

        </div>);
      }else{
        contents = (<ButtonBar>
          {newButton}
        </ButtonBar>);
      }


      return (
        <div >
          {indexBar}
          {contents}
        </div>
      );

    }else{
      // We are editing
      return (
        <Editor article={this.state.currentArticle} complete={(newArticle)=>{this.handleEditorReturn(newArticle)}}/>
      )
    }


  }
}


export default ContentArea;
